@@ -1,6 +1,6 @@
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 *
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
@@ -20,6 +20,7 @@
 
 #include "oxygengtkcellinfo.h"
 #include "oxygengeometry.h"
+#include "oxygengtkutils.h"
 
 #include <iostream>
 #include <cassert>
@@ -200,7 +201,7 @@ namespace Oxygen
     //____________________________________________________________________________
     GdkRectangle Gtk::CellInfo::backgroundRect( GtkTreeView* treeView ) const
     {
-        GdkRectangle out = {0, 0, -1, -1 };
+        GdkRectangle out( Gtk::gdk_rectangle() );
         if( treeView && isValid() )
         { gtk_tree_view_get_background_area( treeView, _path, _column, &out ); }
 
@@ -226,7 +227,7 @@ namespace Oxygen
         */
         _isLast = std::vector<bool>(_depth, false);
 
-        unsigned int index( _depth-1 );
+        int index( _depth-1 );
         for( CellInfo parent = cellInfo; parent.isValid() && parent.depth() > 0; parent = parent.parent() )
         {
             assert( index >= 0 );
