@@ -23,6 +23,7 @@
 #include "oxygencoloreffect.h"
 #include "oxygencolorutils.h"
 #include "oxygenfontinfo.h"
+#include "oxygenshadowhelper.h"
 #include "oxygentimeline.h"
 #include "config.h"
 
@@ -39,6 +40,11 @@
 #include <iostream>
 #include <sstream>
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#include <X11/Xatom.h>
+#endif
+
 namespace Oxygen
 {
 
@@ -52,6 +58,8 @@ namespace Oxygen
     from the oxygenrc file provided with oxygen-gtk
     */
     QtSettings::QtSettings( void ):
+        _wmShadowsSupported( false ),
+        _wmClientSideDecorationSupported( false ),
         _kdeIconTheme( "oxygen" ),
         _kdeFallbackIconTheme( "gnome" ),
         _inactiveChangeSelectionColor( false ),
@@ -123,6 +131,26 @@ namespace Oxygen
         // keep track of whats changed
         bool changed( false );
 
+        // support for wm shadows
+        {
+            const bool wmShadowsSupported( isAtomSupported( ShadowHelper::netWMShadowAtomName ) );
+            if( wmShadowsSupported != _wmShadowsSupported )
+            {
+                _wmShadowsSupported = wmShadowsSupported;
+                changed |= true;
+            }
+        }
+
+        // support for client side decoratiosn
+        {
+            const bool wmClientSideDecorationSupported( isAtomSupported( "_GTK_FRAME_EXTENTS" ) );
+            if( wmClientSideDecorationSupported != _wmClientSideDecorationSupported )
+            {
+                _wmClientSideDecorationSupported = wmClientSideDecorationSupported;
+                changed |= true;
+            }
+        }
+
         // configuration path
         {
             const PathList old( _kdeConfigPathList );
@@ -242,6 +270,74 @@ namespace Oxygen
 
     }
 
+    //_______________________________________________________
+    bool QtSettings::isAtomSupported( const std::string& atomNameQuery ) const
+    {
+
+        // create atom
+        #ifdef GDK_WINDOWING_X11
+
+        // get screen and check
+        GdkScreen* screen = gdk_screen_get_default();
+        if( !screen ) return false;
+
+        // get display and check
+        GdkDisplay *gdkDisplay( gdk_screen_get_display( screen ) );
+        if( !( gdkDisplay && GDK_IS_X11_DISPLAY( gdkDisplay ) ) ) return false;
+        Display* display( GDK_DISPLAY_XDISPLAY( gdkDisplay ) );
+
+        // create atom
+        Atom netSupportedAtom( XInternAtom( display, "_NET_SUPPORTED", False) );
+        if( !netSupportedAtom ) return false;
+
+        // root window
+        Window root( GDK_WINDOW_XID( gdk_screen_get_root_window( screen ) ) );
+        if( !root ) return false;
+
+        Atom type;
+        int format;
+        unsigned char *data;
+        unsigned long count;
+        unsigned long after;
+        int length = 32768;
+
+        while( true )
+        {
+
+            // get atom property on root window
+            // length is incremented until after is zero
+            if( XGetWindowProperty(
+                display, root,
+                netSupportedAtom, 0l, length,
+                false, XA_ATOM, &type,
+                &format, &count, &after, &data) != Success ) return false;
+
+            if( after == 0 ) break;
+
+            // free data, increase length
+            XFree( data );
+            length *= 2;
+            continue;
+
+        }
+
+        Atom* atoms = reinterpret_cast<Atom*>( data );
+        bool found( false );
+        for( unsigned long i = 0; i<count && !found; i++ )
+        {
+            char* atomName = XGetAtomName( display, atoms[i]);
+            if( strcmp( atomName, atomNameQuery.c_str() ) == 0 ) found = true;
+            XFree( atomName );
+        }
+
+        return found;
+
+        #else
+        return false;
+        #endif
+
+    }
+
     //_________________________________________________________
     bool QtSettings::loadOxygen( void )
     {
@@ -725,6 +821,7 @@ namespace Oxygen
             "GtkImage, "
             "GtkInfoBar, "
             "GtkLabel, "
+            "GtkLayout, "
             "GtkMenuItem, "
             "GtkOverlay, "
             "GtkRevealer, "
@@ -832,12 +929,16 @@ namespace Oxygen
         _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::Disabled, Palette::WindowText ) ) );
 
         // tooltips
-        _css.addSection( "GtkWindow#gtk-tooltip" );
+        _css.addSection( ".tooltip" );
         _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, _palette.color( Palette::Tooltip ) ) );
-
-        _css.addSection( "GtkWindow#gtk-tooltip GtkLabel" );
         _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, _palette.color( Palette::TooltipText ) ) );
-        addLinkColors( "GtkWindow#gtk-tooltip GtkLabel", "[Colors:Tooltip]" );
+
+        _css.addSection( ".tooltip *" );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BACKGROUND_COLOR, "transparent" ) );
+        _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_COLOR, "inherit" ) );
+
+        addLinkColors( ".tooltip", "[Colors:Tooltip]" );
+        addLinkColors( ".tooltip *", "[Colors:Tooltip]" );
 
         // rubber band selection (at least for nautilus)
         // FIXME: is Palette::Selected the proper color? Is 0.35 the proper alpha?
@@ -1085,19 +1186,19 @@ namespace Oxygen
 
         // menubar animation type
         std::string menuBarAnimationType( _oxygen.getValue( "[Style]", "MenuBarAnimationType", "MB_FADE") );
-        if( menuBarAnimationType == "MB_NONE" ) _menuBarAnimationType = None;
+        if( menuBarAnimationType == "MB_NONE" ) _menuBarAnimationType = NoAnimation;
         else if( menuBarAnimationType == "MB_FADE" ) _menuBarAnimationType = Fade;
         else if( menuBarAnimationType == "MB_FOLLOW_MOUSE" ) _menuBarAnimationType = FollowMouse;
 
         // menubar animation type
         std::string menuAnimationType( _oxygen.getValue( "[Style]", "MenuAnimationType", "ME_FADE") );
-        if( menuAnimationType == "ME_NONE" ) _menuAnimationType = None;
+        if( menuAnimationType == "ME_NONE" ) _menuAnimationType = NoAnimation;
         else if( menuAnimationType == "ME_FADE" ) _menuAnimationType = Fade;
         else if( menuAnimationType == "ME_FOLLOW_MOUSE" ) _menuAnimationType = FollowMouse;
 
         // toolbar animation type
         std::string toolBarAnimationType( _oxygen.getValue( "[Style]", "ToolBarAnimationType", "TB_FADE") );
-        if( toolBarAnimationType == "TB_NONE" ) _toolBarAnimationType = None;
+        if( toolBarAnimationType == "TB_NONE" ) _toolBarAnimationType = NoAnimation;
         else if( toolBarAnimationType == "TB_FADE" ) _toolBarAnimationType = Fade;
         else if( toolBarAnimationType == "TB_FOLLOW_MOUSE" ) _toolBarAnimationType = FollowMouse;
 
@@ -1245,6 +1346,46 @@ namespace Oxygen
             Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "0px 12px 0px 0px" ):
             Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_PADDING, "0px 0px 0px 12px" ) );
 
+        // CSD titlebar and shadow
+        setupCssShadows( ".window-frame", true );
+        setupCssShadows( ".window-frame.csd.popup", !_wmShadowsSupported );
+
+        // always disable tooltips because it appears broken
+        setupCssShadows( ".window-frame.csd.tooltip", false );
+
+    }
+
+    //_________________________________________________________
+    void QtSettings::setupCssShadows( const std::string& section, bool enabled )
+    {
+
+        // CSD titlebar and shadows
+        if( enabled )
+        {
+            // default shadows (copied from Adwaita)
+            _css.addSection( section );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_RADIUS, "4px 4px 0 0" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_WIDTH, "0" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_MARGIN, "10px" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "box-shadow", "0 3px 9px 1px rgba(0, 0, 0, 0.5), 0 0 0 1px rgba(0, 0, 0, 0.23);" ) );
+
+            _css.addSection( section + ":backdrop" );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "box-shadow", "0 2px 6px 2px rgba(0, 0, 0, 0.2), 0 0 0 1px rgba(0, 0, 0, 0.18);" ) );
+
+        } else {
+
+            _css.addSection( section );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_RADIUS, "0" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_WIDTH, "0" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_BORDER_STYLE, "none" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( GTK_STYLE_PROPERTY_MARGIN, "0" ) );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "box-shadow", "none" ) );
+
+            _css.addSection( section + ":backdrop" );
+            _css.addToCurrentSection( Gtk::CSSOption<std::string>( "box-shadow", "none" ) );
+
+        }
+
     }
 
     //_________________________________________________________
