@@ -44,12 +44,25 @@ namespace Oxygen
 
         public:
 
+        //!@name property names
+        //@{
+        static const char* const netWMShadowAtomName;
+        //@}
+
         //! constructor
         ShadowHelper( void );
 
         //! destructor
         virtual ~ShadowHelper( void );
 
+        //! true is supported
+        void setSupported( bool value )
+        { _supported = value; }
+
+        //! true if supported
+        bool isSupported( void ) const
+        { return _supported; }
+
         //! reset
         void reset( void );
 
@@ -106,6 +119,9 @@ namespace Oxygen
 
         private:
 
+        //! true if shadows are supported
+        bool _supported;
+
         //! shadow size
         int _size;
 
